@@ -45,7 +45,7 @@ reg_coef_psm <- function(data, formula, dist) {
 #' @param ... Additional arguments passed to \code{censboot}, such as \code{parallel} for parallel computations. See \code{?boot::censboot} for details.
 #'
 #' @return A data frame containing coefficient estimates, bootstrap confidence intervals, and bootstrap p-values.
-#' @details p-values can be computed by inverting the corresponding confidence intervals, as described in Section 12.2 of Thulin (2021) and Section 3.12 in Hall (1992). This function computes p-values in this way from "coxph" or "survreg" objects. The approach relies on the fact that:
+#' @details p-values can be computed by inverting the corresponding confidence intervals, as described in Section 14.2 of Thulin (2024) and Section 3.12 in Hall (1992). This function computes p-values in this way from "coxph" or "survreg" objects. The approach relies on the fact that:
 #' - the p-value of the two-sided test for the parameter theta is the smallest alpha such that theta is not contained in the corresponding 1-alpha confidence interval,
 #' - for a test of the parameter theta with significance level alpha, the set of values of theta that aren't rejected by the two-sided test (when used as the null hypothesis) is a 1-alpha confidence interval for theta.
 #' @importFrom Rdpack reprompt
@@ -167,6 +167,7 @@ censboot_summary <- function(model,
   }
 
   # Compute p-values:
+  if(is.null(pval_precision)) { pval_precision <- 1/R }
   for(i in 1:p)
   {
     results[i, 4] <- boot.pval(boot_res,
@@ -182,8 +183,8 @@ censboot_summary <- function(model,
     results$`Adjusted p-value` <- round(stats::p.adjust(results[, 4], method = adjust.method), round(log10(R)))
   }
   # Round p-values:
-  results[,4] <- round(results[,4], round(log10(R)))
-
+  results$p.value[results$p.value != pval_precision] <- round(results$p.value[results$p.value != pval_precision], round(log10(R)))
+  results$p.value[results$p.value == pval_precision] <- paste0("<", round(results$p.value[results$p.value == pval_precision], round(log10(R))))
 
   return(results)
 
